{***************************************************}
{* 4-to-2 encoder unit for LogicGates6 application *}
{***************************************************}

unit encoder_4to2;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, Menus, common;

type
  {*********}
  { TfEnc42 }
  {*********}
  TfEnc42 = class(TForm)
    mMenu: TMainMenu;
    mSettings, mSettingsSwitchesOff, mSettingsNoWarnings: TMenuItem;
    stTitle: TStaticText;
    imGate1, imGate2: TImage;
    imSwitch0, imSwitch1, imSwitch2, imSwitch3: TImage;
    shSwitch0a, shSwitch0b, shSwitch1a, shSwitch1b: TShape;
    shSwitch2a, shSwitch2b, shSwitch3a, shSwitch3b: TShape;
    shLedA0, shLedA1, shLedY0, shLedY1, shLedY2, shLedY3: TShape;
    stDetails, stBoolean: TStaticText;
    stA0, stA1, stY0, stY1, stY2, stY3: TStaticText;
    laIn, laOut, laA0, laA1, laY0, laY1, laY2, laY3: TLabel;
    Label1, Label2, Label3, Label4, Label5, Label10, Label11, Label12, Label13, Label14, Label15: TLabel;
    Label17, Label18, Label19, Label20, Label21, Label23, Label24, Label25, Label26, Label27: TLabel;
    Label28, Label29, Label6, Label7, Label8, Label9, Label16, Label22: TLabel;
    Shape1, Shape2, Shape3, Shape4, Shape5, Shape6, Shape7, Shape8, Shape21: TShape;
    Shape22, Shape23, Shape24, Shape25, Shape26, Shape27, Shape28, Shape29, Shape30, Shape37: TShape;
    Shape10, Shape11, Shape12, Shape13, Shape14, Shape31, Shape32, Shape33, Shape34, Shape35: TShape;
    Shape36, Shape38, Shape39, Shape40, Shape41, Shape42, Shape43, Shape44, Shape9: TShape;
    Memo1: TMemo;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure mSettingsSwitchesOffClick(Sender: TObject);
    procedure mSettingsNoWarningsClick(Sender: TObject);
    procedure imSwitch0Click(Sender: TObject);
    procedure imSwitch1Click(Sender: TObject);
    procedure imSwitch2Click(Sender: TObject);
    procedure imSwitch3Click(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    aInput: array[0..3] of Boolean;                                            // input lines status (variables)
    aOutput: array[0..1] of Boolean;                                           // output lines status (variables)
    aSwitches: array[0..3] of TSwitch;                                         // input lines switches
    aLedsInput: array[0..3] of TLed;                                           // input lines LEds
    aLedsOutput: array[0..1] of TLed;                                          // output lines LEDs
    bSwitchesOff, bNoWarnings: Boolean;
  end;

var
  fEnc42: TfEnc42;

implementation

{$R *.lfm}

{*********}
{ TfEnc42 }
{*********}

{ Application start: Init the LED and Switch records (would be object creation in OOP) }

procedure TfEnc42.FormCreate(Sender: TObject);

var
  I: Integer;

begin
  // Apply subscripts
  stY3.Caption := StringReplace(stY3.Caption, '3', SUB_Numbers[3], []); stY2.Caption := StringReplace(stY2.Caption, '2', SUB_Numbers[2], []);
  stY1.Caption := StringReplace(stY1.Caption, '1', SUB_Numbers[1], []); stY0.Caption := StringReplace(stY0.Caption, '0', SUB_Numbers[0], []);
  stA1.Caption := StringReplace(stA1.Caption, '1', SUB_Numbers[1], []); stA0.Caption := StringReplace(stA0.Caption, '0', SUB_Numbers[0], []);
  laY3.Caption := StringReplace(laY3.Caption, '3', SUB_Numbers[3], []); laY2.Caption := StringReplace(laY2.Caption, '2', SUB_Numbers[2], []);
  laY1.Caption := StringReplace(laY1.Caption, '1', SUB_Numbers[1], []); laY0.Caption := StringReplace(laY0.Caption, '0', SUB_Numbers[0], []);
  laA1.Caption := StringReplace(laA1.Caption, '1', SUB_Numbers[1], []); laA0.Caption := StringReplace(laA0.Caption, '0', SUB_Numbers[0], []);
  for I := 0 to 3 do begin
    stDetails.Caption := StringReplace(stDetails.Caption, 'Y' + IntToStr(I), 'Y' + SUB_Numbers[I], [rfReplaceAll]);
    stBoolean.Caption := StringReplace(stBoolean.Caption, IntToStr(I), SUB_Numbers[I], [rfReplaceAll]);
  end;
  // Create arrays with switches and LEds
  aLedsInput[0].LedShape := shLedY0; aLedsInput[1].LedShape := shLedY1;
  aLedsInput[2].LedShape := shLedY2; aLedsInput[3].LedShape := shLedY3;
  aLedsOutput[0].LedShape := shLedA0; aLedsOutput[1].LedShape := shLedA1;
  with aSwitches[0] do begin
    SwitchShapeA := shSwitch0a; SwitchShapeB := shSwitch0b;
    SwitchShapeAOffPos := 465;                                                 // vertical position (Top property) of open switch
  end;
  with aSwitches[1] do begin
    SwitchShapeA := shSwitch1a; SwitchShapeB := shSwitch1b;
    SwitchShapeAOffPos := 344;
  end;
  with aSwitches[2] do begin
    SwitchShapeA := shSwitch2a; SwitchShapeB := shSwitch2b;
    SwitchShapeAOffPos := 214;
  end;
  with aSwitches[3] do begin
    SwitchShapeA := shSwitch3a; SwitchShapeB := shSwitch3b;
    SwitchShapeAOffPos := 96;
  end;
  // Default setings for switch auto-close and warning messages
  bSwitchesOff := False; bNoWarnings := False;
end;

{ Window show-up: Init the encoder circuit }

procedure TfEnc42.FormActivate(Sender: TObject);

var
  I: Integer;

begin
  // Input line 0 active High
  SwitchOn(aSwitches[0]); LedOn(aLedsInput[0]); aInput[0] := True;
  // All other input lines Low
  for I := 1 to 3 do begin
    SwitchOff(aSwitches[I]);
    LedOff(aLedsInput[I]);
    aInput[I] := False;
  end;
  // All outputs Low
  for I := 0 to 1 do begin
    LedOff(aLedsOutput[I]);
    aOutput[I] := False;
  end;
end;

{ Menu item "Settings > Automatically push switches off": Toggle auto switches-off }

procedure TfEnc42.mSettingsSwitchesOffClick(Sender: TObject);

begin
  if mSettingsSwitchesOff.Checked then
    mSettingsSwitchesOff.Checked := False
  else
    mSettingsSwitchesOff.Checked := True;
  bSwitchesOff := mSettingsSwitchesOff.Checked;
end;

{ Menu item "Settings > Do not display warning messages": Toggle warnings display }

procedure TfEnc42.mSettingsNoWarningsClick(Sender: TObject);

begin
  if mSettingsNoWarnings.Checked then
    mSettingsNoWarnings.Checked := False
  else
    mSettingsNoWarnings.Checked := True;
  bNoWarnings := mSettingsNoWarnings.Checked;
end;

{ User click on switches: Set corr. input line High or Low; do 4-to-2 encoder simulation }

procedure TfEnc42.imSwitch0Click(Sender: TObject);

begin
  SwitchOnOff(0, True, not aSwitches[0].SwitchOn, aSwitches, aLedsInput, aInput, bSwitchesOff, bNoWarnings);
  Encoder42(aInput, aOutput, aLedsOutput);
end;

procedure TfEnc42.imSwitch1Click(Sender: TObject);

begin
  SwitchOnOff(1, True, not aSwitches[1].SwitchOn, aSwitches, aLedsInput, aInput, bSwitchesOff, bNoWarnings);
  Encoder42(aInput, aOutput, aLedsOutput);
end;

procedure TfEnc42.imSwitch2Click(Sender: TObject);

begin
  SwitchOnOff(2, True, not aSwitches[2].SwitchOn, aSwitches, aLedsInput, aInput, bSwitchesOff, bNoWarnings);
  Encoder42(aInput, aOutput, aLedsOutput);
end;

procedure TfEnc42.imSwitch3Click(Sender: TObject);

begin
  SwitchOnOff(3, True, not aSwitches[3].SwitchOn, aSwitches, aLedsInput, aInput, bSwitchesOff, bNoWarnings);
  Encoder42(aInput, aOutput, aLedsOutput);
end;

{Button "Close": Close the simulation window }

procedure TfEnc42.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

