{*****************************************}
{* Main unit for LogicGates6 application *}
{*****************************************}

unit logic6;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, Buttons, Menus,
  encoder_4to2, encoder_8to3, encoder_4to2pri, decoder_2to4, multiplexer_4x1, demultiplexer_1x4, common;

type
  {**********}
  { TfLogic6 }
  {**********}
  TfLogic6 = class(TForm)
    mMenu: TMainMenu;
    mFile, mFileExit, mHelp, mHelpAbout: TMenuItem;
    StaticText1: TStaticText;
    Label1: TLabel;
    rbCircuit0, rbCircuit1, rbCircuit2, rbCircuit3: TRadioButton;
    rbCircuit4, rbCircuit5: TRadioButton;
    Shape1: TShape;
    stDetails: TStaticText;
    imBlockDiagram: TImage;
    btShow: TButton;
    procedure FormCreate(Sender: TObject);
    procedure mFileExitClick(Sender: TObject);
    procedure mHelpAboutClick(Sender: TObject);
    procedure btShowClick(Sender: TObject);
    procedure rbCircuit0Change(Sender: TObject);
    procedure rbCircuit1Change(Sender: TObject);
    procedure rbCircuit2Change(Sender: TObject);
    procedure rbCircuit3Change(Sender: TObject);
    procedure rbCircuit4Change(Sender: TObject);
    procedure rbCircuit5Change(Sender: TObject);
  private
    iCircuit: Integer;
  end;

var
  fLogic6: TfLogic6;

implementation

{$R *.lfm}

{ Display details for actual circuit }

procedure ShowDetails(Circuit: Integer);

const
  BlockDiagrams: array[0..5] of string = (
    'encoder_4to2', 'encoder_8to3', '', 'decoder_2to4', 'multiplexer_4x1', 'demultiplexer_1x4'
  );
  Description: array[0..5, 0..2] of AnsiString = (
    ( 'An encoder is a combinational circuit that has a maximum of 2n input lines and n output lines. It will produce a  binary code equivalent to the input, which is active High. Therefore, the encoder encodes 2n input lines with n bits.',
      '',
      'A 4-to-2 encoder has four inputs Y3, Y2, Y1, Y0 and two outputs A1, A0.'
    ),
    ( 'An encoder is a combinational circuit that has maximum of  2n input lines and  n output lines. It will produce a binary code equivalent to the input, which is active High. Therefore, the encoder encodes 2n input lines with n bits.',
      '',
      'A 8-to-3 encoder has eight inputs Y7, Y6, Y5, Y4, Y3, Y2, Y1, Y0 and three outputs A2, A1, A0. In fact, this circuit is an octal to binary encoder.'
    ),
    ( 'Normal encoders have two drawbacks: 1. ambiguity when all outputs are equal to zero  (Y0 being  active High  OR all inputs being  Low);  2. possible  incorrect code when more than one input is active High.',
      'These  drawbacks may be overcome by using a  priority encoder. The first  issue is solved by a  supplementary  output  V, set to  1 if the  input is valid (not all inputs Low).  To resolve the second one, a priority is assigned to each input in a way that the  output will be the  binary code corresponding to the active High input, which has higher priority.',
      'A 4-to-2 priority encoder has four inputs Y3, Y2, Y1, Y0 and three outputs A1, A0, V.'
    ),
    ( 'A decoder is a  combinational circuit  that has  n input lines and a  maximum of  2n output lines. One of these outputs will be active High based on the combination of inputs present,  but this only if the decoder is  enabled (input E is  active High). The outputs of a decoder are nothing but the min terms of n input variables',
      '',
      'A 2-to-4 decoder has three inputs A1, A0, E and four outputs Y3, Y2, Y1, Y0.'
    ),
    ( 'A  multiplexer is a  combinational circuit that has a  maximum of  2n data inputs,  n selection lines and one single output line. One of the data inputs will be connected to the output, based on the values of the selection lines.',
      'As there are  n selection lines, there will be  2n possible combinations of  zeros and ones. So, each combination will select exactly one data input.',
      'A 4x1 multiplexer has four data inputs I3, I2, I1, I0,  two selection lines S1, S0 and one output Y.'
    ),
    ( 'A  demultiplexer is a  combinational circuit, that has one single input,  n selection lines and a  maximum of  2n outputs.  The input will be connected to one of these outputs, based on the values of the selection lines.',
      'As there are n selection lines,  there will be 2n possible combinations of zeros and ones. So, each combination will select exactly one output.',
      'A 1x4 demultiplexer has one input I, two selection lines, S1, S0 and four outputs Y3, Y2, Y1, Y0.'
    )
  );

var
  I: Integer;
  FileName: string;

begin
  // Display circuit details
  fLogic6.stDetails.Caption := '';
  for I := 0 to 2 do begin
    if Description[Circuit, I] <> '' then
      fLogic6.stDetails.Caption := fLogic6.stDetails.Caption + Description[Circuit, I] + LineEnding + LineEnding;
  end;
  // Display circuit blockdiagram
  if BlockDiagrams[Circuit] = '' then begin
    fLogic6.imBlockDiagram.Picture.Clear;
  end
  else begin
    FileName := './img/' + BlockDiagrams[Circuit] + '.jpg'; DoDirSeparators(FileName);
    fLogic6.imBlockDiagram.Picture.LoadFromFile(FileName);
  end;
  // Apply subscripts and superscripts
  for I := 0 to 9 do begin
    fLogic6.stDetails.Caption := StringReplace(fLogic6.stDetails.Caption, 'A' + IntToStr(I), 'A' + SUB_Numbers[I], [rfReplaceAll]);
    fLogic6.stDetails.Caption := StringReplace(fLogic6.stDetails.Caption, 'I' + IntToStr(I), 'I' + SUB_Numbers[I], [rfReplaceAll]);
    fLogic6.stDetails.Caption := StringReplace(fLogic6.stDetails.Caption, 'S' + IntToStr(I), 'S' + SUB_Numbers[I], [rfReplaceAll]);
    fLogic6.stDetails.Caption := StringReplace(fLogic6.stDetails.Caption, 'Y' + IntToStr(I), 'Y' + SUB_Numbers[I], [rfReplaceAll]);
    fLogic6.stDetails.Caption := StringReplace(fLogic6.stDetails.Caption, '2n', '2ⁿ', [rfReplaceAll]);
  end;
end;

{**********}
{ TfLogic6 }
{**********}

{ Application start: Initialisation }

procedure TfLogic6.FormCreate(Sender: TObject);

begin
  // Display 4-to-2 encoder at application start
  iCircuit := 0;
  ShowDetails(iCircuit);
end;

{ Menu item "File > Exit": Exit application }

procedure TfLogic6.mFileExitClick(Sender: TObject);

begin
  Close;
end;

{ Menu item "Help > About": Display application about }

procedure TfLogic6.mHelpAboutClick(Sender: TObject);

var
  S: string;

begin
  S := 'Electronics simulation.' + LineEnding;
  S += 'Common logic gates based electronics: Combinational circuits.' + LineEnding + LineEnding;
  S += 'Version 1.0, © allu, September 2020.';
  MessageDlg('About "LogicGates6"', S, mtInformation, [mbOK], 0);
end;

{ Button "Show": Open simulation window for actual circuit }

procedure TfLogic6.btShowClick(Sender: TObject);

begin
  case iCircuit of
    0: fEnc42.ShowModal;
    1: fEnc83.ShowModal;
    2: fEnc42p.ShowModal;
    3: fDec24.ShowModal;
    4: fMult41.ShowModal;
    5: fDemult14.ShowModal;
  end;
end;

{ New circuit selected by user (radiobutton changes): Display circuit details }

procedure TfLogic6.rbCircuit0Change(Sender: TObject);

begin
  if rbCircuit0.Checked then begin
    iCircuit := 0;
    ShowDetails(iCircuit);
  end;
end;

procedure TfLogic6.rbCircuit1Change(Sender: TObject);

begin
  if rbCircuit1.Checked then begin
    iCircuit := 1;
    ShowDetails(iCircuit);
  end;
end;

procedure TfLogic6.rbCircuit2Change(Sender: TObject);

begin
  if rbCircuit2.Checked then begin
    iCircuit := 2;
    ShowDetails(iCircuit);
  end;
end;

procedure TfLogic6.rbCircuit3Change(Sender: TObject);

begin
  if rbCircuit3.Checked then begin
    iCircuit := 3;
    ShowDetails(iCircuit);
  end;
end;

procedure TfLogic6.rbCircuit4Change(Sender: TObject);

begin
  if rbCircuit4.Checked then begin
    iCircuit := 4;
    ShowDetails(iCircuit);
  end;
end;

procedure TfLogic6.rbCircuit5Change(Sender: TObject);

begin
  if rbCircuit5.Checked then begin
    iCircuit := 5;
    ShowDetails(iCircuit);
  end;
end;

end.

