{****************************************************}
{* 4x1 multiplexer unit for LogicGates6 application *}
{****************************************************}

unit multiplexer_4x1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, Menus, common;

type
  {**********}
  { TfMult41 }
  {**********}
  TfMult41 = class(TForm)
    mMenu: TMainMenu;
    mSettings, mSettingsColored: TMenuItem;
    stTitle: TStaticText;
    imGate1, imGate2, imGate3, imGate4, imGate5, imInv1, imInv2: TImage;
    imSwitch0, imSwitch1: TImage;
    shSwitch0a, shSwitch0b, shSwitch1a, shSwitch1b: TShape;
    shLedS0, shLedS1, shLedI0, shLedI1, shLedI2, shLedI3: TShape;
    shLedY0, shLedY1, shLedY2, shLedY3, shLedY: TShape;
    stS0, stS1, stY: TStaticText;
    stDetails, stExpl, stBoolean: TStaticText;
    laI0, laI1, laI2, laI3, laY0, laY1, laY2, laY3: TLabel;
    laSel, laOut, laS0, laS1, laY: TLabel;
    laI00, laI01, laI02, laI03: TLabel;
    Label29, Label48, Label14, Label20, Label21, Label26, Label27, Label28, Label3: TLabel;
    Label38, Label39, Label4, Label42, Label43, Label44, Label45, Label46, Label47: TLabel;
    Shape1, Shape10, Shape11, Shape12, Shape13, Shape14, Shape15, Shape16, Shape17, Shape18, Shape19: TShape;
    Shape20, Shape21, Shape22, Shape23, Shape24, Shape25, Shape26, Shape27, Shape28, Shape29: TShape;
    Shape3, Shape30, Shape31, Shape32, Shape33, Shape34, Shape35, Shape36, Shape37, Shape38, Shape39: TShape;
    Shape4, Shape40, Shape41, Shape42, Shape43, Shape44, Shape45, Shape46, Shape47, Shape48, Shape49: TShape;
    Shape50, Shape51, Shape52, Shape53, Shape54, Shape55, Shape56, Shape59: TShape;
    Shape6, Shape60, Shape61, Shape62, Shape63, Shape64, Shape65, Shape66, Shape67, Shape7, Shape8, Shape9: TShape;
    Memo1: TMemo;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure mSettingsColoredClick(Sender: TObject);
    procedure imSwitch0Click(Sender: TObject);
    procedure imSwitch1Click(Sender: TObject);
    procedure shLedI0MouseDown(Sender: TObject);
    procedure shLedI1MouseDown(Sender: TObject);
    procedure shLedI2MouseDown(Sender: TObject);
    procedure shLedI3MouseDown(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    aSelection: array[0..1] of Boolean;                                        // selection lines status (variables)
    aInput: array[0..3] of Boolean;                                            // input lines status (variables)
    bOutput: Boolean;                                                          // output line status (variables)
    iLine: Integer;                                                            // input line selected (0..3)
    aSwitches: array[0..1] of TSwitch;                                         // selection lines switches
    aLedsSelection: array[0..1] of TLed;                                       // selection lines LEDs
    aLedsInput: array[0..3] of TLed;                                           // input lines LEDs (also used as switches)
    aLedsOutput: array[0..3] of TLed;                                          // AND gates output lines LEDs
    LedOutput: TLed;                                                           // output line LED
    bColoredLeds: Boolean;
  end;

const
  // Input LEDs and AND gates output LEDs default color
  InputLedDefaultColor: TColor = clLime;
  OutputLedDefaultColor: TColor = $00B0FFFF;

var
  fMult41: TfMult41;

implementation

{$R *.lfm}

{**********}
{ TfMult41 }
{**********}

{ Application start: Init the LED and Switch records }

procedure TfMult41.FormCreate(Sender: TObject);

var
  I: Integer;

begin
  // Apply subscripts
  stS1.Caption := StringReplace(stS1.Caption, '1', SUB_Numbers[1], []); stS0.Caption := StringReplace(stS0.Caption, '0', SUB_Numbers[0], []);
  laI3.Caption := StringReplace(laI3.Caption, '3', SUB_Numbers[3], []); laI2.Caption := StringReplace(laI2.Caption, '2', SUB_Numbers[2], []);
  laI1.Caption := StringReplace(laI1.Caption, '1', SUB_Numbers[1], []); laI0.Caption := StringReplace(laI0.Caption, '0', SUB_Numbers[0], []);
  laY3.Caption := StringReplace(laY3.Caption, '3', SUB_Numbers[3], []); laY2.Caption := StringReplace(laY2.Caption, '2', SUB_Numbers[2], []);
  laY1.Caption := StringReplace(laY1.Caption, '1', SUB_Numbers[1], []); laY0.Caption := StringReplace(laY0.Caption, '0', SUB_Numbers[0], []);
  laS1.Caption := StringReplace(laS1.Caption, '1', SUB_Numbers[1], []); laS0.Caption := StringReplace(laS0.Caption, '0', SUB_Numbers[0], []);
  laI03.Caption := StringReplace(laI03.Caption, '3', SUB_Numbers[3], []); laI02.Caption := StringReplace(laI02.Caption, '2', SUB_Numbers[2], []);
  laI01.Caption := StringReplace(laI01.Caption, '1', SUB_Numbers[1], []); laI00.Caption := StringReplace(laI00.Caption, '0', SUB_Numbers[0], []);
  for I := 0 to 3 do begin
    stDetails.Caption := StringReplace(stDetails.Caption, 'I' + IntToStr(I), 'I' + SUB_Numbers[I], [rfReplaceAll]);
    stDetails.Caption := StringReplace(stDetails.Caption, 'S' + IntToStr(I), 'S' + SUB_Numbers[I], [rfReplaceAll]);
    stExpl.Caption := StringReplace(stExpl.Caption, IntToStr(I), SUB_Numbers[I], [rfReplaceAll]);
    stBoolean.Caption := StringReplace(stBoolean.Caption, IntToStr(I), SUB_Numbers[I], [rfReplaceAll]);
  end;
  // Create arrays with switches and LEds
  aLedsSelection[0].LedShape := shLedS0; aLedsSelection[1].LedShape := shLedS1;
  aLedsInput[0].LedShape := shLedI0; aLedsInput[1].LedShape := shLedI1;
  aLedsInput[2].LedShape := shLedI2; aLedsInput[3].LedShape := shLedI3;
  aLedsOutput[0].LedShape := shLedY0; aLedsOutput[1].LedShape := shLedY1;
  aLedsOutput[2].LedShape := shLedY2; aLedsOutput[3].LedShape := shLedY3;
  LedOutput.LedShape := shLedY;
  with aSwitches[0] do begin
    SwitchShapeA := shSwitch0a; SwitchShapeB := shSwitch0b;
    SwitchShapeAOffPos := 214;                                                 // vertical position (Top property) of open switch
  end;
  with aSwitches[1] do begin
    SwitchShapeA := shSwitch1a; SwitchShapeB := shSwitch1b;
    SwitchShapeAOffPos := 89;
  end;
  // No colored LEDs at application start
  bColoredLeds := False;
end;

{ Window show-up: Init the multiplexer circuit }

procedure TfMult41.FormActivate(Sender: TObject);

var
  I: Integer;

begin
  // All selection lines Low
  for I := 0 to 1 do begin
    SwitchOff(aSwitches[I]);
    LedOff(aLedsSelection[I]);
    aSelection[I] := False;
  end;
  // Input LEDs and AND gates output LEDs color
  SetLedColors(InputLedDefaultColor, bColoredLeds, aLedsInput);
  SetLedColors(OutputLedDefaultColor, bColoredLeds, aLedsOutput);
  // All input lines and AND gates output lines Low
  for I := 0 to 3 do begin
    LedOff(aLedsInput[I]);
    LedOff(aLedsOutput[I]);
    aInput[I] := False;
  end;
  // Input line actually selected
  iLine := 0;
  // Output line LED color
  SetLedColor(iLine, clYellow, bColoredLeds, LedOutput);
  // Output line Low
  LedOff(LedOutput); bOutput := False;
end;

{ Menu item "Settings > Use colored input LEDs": Toggle colored/not colored input LEDs }

procedure TfMult41.mSettingsColoredClick(Sender: TObject);

begin
  if mSettingsColored.Checked then
    mSettingsColored.Checked := False
  else
    mSettingsColored.Checked := True;
  bColoredLeds := mSettingsColored.Checked;
  // Input lines LEDs color; use this color also for AND gates output LEDs and output LED
  // This is a simple way to show, which input is actually selected and thus connected to the output
  SetLedColors(InputLedDefaultColor, bColoredLeds, aLedsInput);
  SetLedColors(OutputLedDefaultColor, bColoredLeds, aLedsOutput);
  SetLedColor(iLine, OutputLedDefaultColor, bColoredLeds, LedOutput);
end;

{ User click on selection line switches: Set corr. selection line High or Low; do 4x1 multiplexer simulation }

procedure TfMult41.imSwitch0Click(Sender: TObject);

begin
  SwitchOnOff(0, True, not aSwitches[0].SwitchOn, aSwitches, aLedsSelection, aSelection, False, True);
  Multiplexer41(aSelection, aInput, iLine, bOutput, aLedsOutput, LedOutput);
  SetLedColor(iLine, clYellow, bColoredLeds, LedOutput);
end;

procedure TfMult41.imSwitch1Click(Sender: TObject);

begin
  SwitchOnOff(1, True, not aSwitches[1].SwitchOn, aSwitches, aLedsSelection, aSelection, False, True);
  Multiplexer41(aSelection, aInput, iLine, bOutput, aLedsOutput, LedOutput);
  SetLedColor(iLine, clYellow, bColoredLeds, LedOutput);
end;

{ User click on input line LEDs ("virtual switches"): Set corr. input line High or Low; do 4x1 multiplexer simulation }

procedure TfMult41.shLedI0MouseDown(Sender: TObject);

begin
  SwitchOnOff(0, False, not aLedsInput[0].LedOn, VirtualSwitches, aLedsInput, aInput, False, True);
  Multiplexer41(aSelection, aInput, iLine, bOutput, aLedsOutput, LedOutput);
  SetLedColor(iLine, clYellow, bColoredLeds, LedOutput);
end;

procedure TfMult41.shLedI1MouseDown(Sender: TObject);

begin
  SwitchOnOff(1, False, not aLedsInput[1].LedOn, VirtualSwitches, aLedsInput, aInput, False, True);
  Multiplexer41(aSelection, aInput, iLine, bOutput, aLedsOutput, LedOutput);
  SetLedColor(iLine, clYellow, bColoredLeds, LedOutput);
end;

procedure TfMult41.shLedI2MouseDown(Sender: TObject);

begin
  SwitchOnOff(2, False, not aLedsInput[2].LedOn, VirtualSwitches, aLedsInput, aInput, False, True);
  Multiplexer41(aSelection, aInput, iLine, bOutput, aLedsOutput, LedOutput);
  SetLedColor(iLine, clYellow, bColoredLeds, LedOutput);
end;

procedure TfMult41.shLedI3MouseDown(Sender: TObject);

begin
  SwitchOnOff(3, False, not aLedsInput[3].LedOn, VirtualSwitches, aLedsInput, aInput, False, True);
  Multiplexer41(aSelection, aInput, iLine, bOutput, aLedsOutput, LedOutput);
  SetLedColor(iLine, clYellow, bColoredLeds, LedOutput);
end;

{Button "Close": Close the simulation window }

procedure TfMult41.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

